@@ -19,11 +19,8 @@ use Text::Editor::Easy;
 
 Text::Editor::Easy->new(
     {
-        'x_offset' => 240,
-        'y_offset' => 150,
-        'width'    => 300,
-        'height'   => 300,
         'focus'    => 'yes',
+        'trace' => { 'all' => 'tmp/' },
     }
 );
 
@@ -35,4 +32,4 @@ print "The user have closed the window\n";
 # in fact more than one thread
 # created. Still, the program seems
 # to dispose of none.
-#
\ No newline at end of file
+#
