@@ -9,11 +9,11 @@ Text::Editor::Easy::Line - Object oriented interface to a file line (managed by
 
 =head1 VERSION
 
-Version 0.41
+Version 0.42
 
 =cut
 
-our $VERSION = '0.41';
+our $VERSION = '0.42';
 
 # Les fonctions de File_manager.pm ralisant toutes les mthodes de ce package commencent par "line_" puis reprennent
 # le nom de la mthode
@@ -57,14 +57,6 @@ sub new {
     return $line;
 }
 
-sub text {
-    my ($self) = @_;
-
-    my $ref    = refaddr $self;
-    my $editor = $ref_Editor{$ref};
-    return $editor->get_text_from_ref( $ref_id{$ref} );
-}
-
 sub next {
     my ($self) = @_;
 
@@ -163,6 +155,9 @@ my %sub = (
     'bottom_ord' => [ 'graphic', \&Text::Editor::Easy::Abstract::line_bottom_ord ],
     'set' => [ 'graphic', \&Text::Editor::Easy::Abstract::line_set ],
     'add_seek_start' => 1,
+	'get_info' => 1,
+	'set_info' => 1,
+    'text' => 1,
 );
 
 
